Type Ship Extends SpaceObject
	Field rotacc: Acceleration
	Field speedacc: Acceleration
	Field lsc: Int
	Field weaponlist: TList
	Field shieldhealth: Float
	Field maxhealth: Float
	Field maxshield: Float
	Field energy: Float
	Field maxenergy: Float
	Field energyps: Float
	Field shieldps: Float
	
	Function create: Ship(x: Float, y: Float, rotation: Float, ..
								health: Float, shieldhealth: Float, collisiondamage: Float, collisionradius: Float, ..
								rotationspeed: Float, rotationlos: Float, maxrotation: Float, ..
								speedacceleration: Float, speedlos: Float, maxspeed: Float, ..
								energy: Float, energyps: Float, shieldps: Float)
		Local s: Ship = New Ship
		
		s.x = x
		s.y = y
		s.rotation = rotation
		
		s.maxhealth = health
		s.health = health
		s.collisiondamage = collisiondamage
		s.collisionradius = collisionradius
		
		s.shieldhealth = shieldhealth
		s.maxshield = shieldhealth
		
		s.energy = energy
		s.maxenergy = energy
		s.energyps = energyps
		s.shieldps = shieldps
		
		s.rotacc.minimum = -maxrotation
		s.rotacc.maximum = maxrotation
		s.rotacc.add = rotationspeed
		s.rotacc.sub = rotationspeed
		s.rotacc.ntns = rotationlos
		
		s.speedacc.maximum = maxspeed
		s.speedacc.sub = speedlos
		s.speedacc.add = speedacceleration
		
		Return s
	End Function
	
	Method getType: Int()
		Return SOT_SHIP
	End Method
	
	Method New()
		rotacc = New Acceleration
		speedacc = New Acceleration
		
		weaponlist = New TList
		
		weaponlist.AddLast(Weapon.create(Self, 11, 24, 0, 255, 0, 2000, 200, 100, 1, 0.125, 10))
		weaponlist.AddLast(Weapon.create(Self, 11, -24, 0, 255, 0, 2000, 200, 100, 1, 0.125, 10))
		MainGame.solist.add(Self)
	End Method
	
	Method collisionPossible: Int(o: SpaceObject)
		Return True
	End Method
	
	Method collided(damage: Float)'p: Point)
		If shieldhealth <= 0
			health :- damage
		Else
			Shield.create(self, x, y, 0.75, 0.75, 0, 0, 255, 2, 0, 0)
			shieldhealth :- damage
			If shieldhealth < 0
			 health :+ shieldhealth
			 shieldhealth = 0
			End If
		End If
	End Method
	
	Method draw(p: Point, wscale: Float, wrotation: Float)
		SetRotation rotation+wrotation
		SetMaskColor 0, 0, 0
		SetScale wscale, wscale
		SetBlend MASKBLEND
		
		DrawImage RessourceLoader.shipimages[0], p.x+x*wscale, p.y+y*wscale
		
		SetScale 1, 1
		SetBlend SOLIDBLEND
		SetRotation 0
	End Method
	
	Method shoot()
		If health > 0
			For Local weap: Weapon = EachIn weaponlist
				weap.shoot(x, y, rotation, speedacc.speed)
			Next
		End If
	End Method
	
	Method rotateLeft(time: Float)
		If health > 0
			rotacc.speeddown(time)
		End if
	End Method
	
	Method rotateRight(time: Float)
		If health > 0
			rotacc.speedup(time)
		End if
	End Method
	
	Method speedUp(time: Float)
		If health > 0
			speedacc.speedup(time)
		End if
	End Method
	
	Method speedDown(time: Float)
		If health > 0
			speedacc.speeddown(time)
		End if
	End Method
	
	Method move(time: Float)
		rotation = rotation + rotacc.speed*time
		x = x + speedacc.speed * Cos(rotation) * time
		y = y + speedacc.speed * Sin(rotation) * time
		rotacc.neartonull(time)
		If speedacc.speed > 0
			If MilliSecs()-lsc > 100000/speedacc.speed/speedacc.speed/speedacc.speed
				Local sp: Spark = Spark.create(x + Cos(rotation-180)*40, y + Sin(rotation-180)*40, 0.5, 0.5, 128, 64, 0, 2, rotation, 0)
				lsc = MilliSecs()
			End If
		End If
		energy :+ energyps*time
		If energy > maxenergy energy = maxenergy
		shieldhealth :+ shieldps*time
		If shieldhealth > maxshield shieldhealth = maxshield 
	End Method
	
	Method destroy()
		Explosion.create(x, y, 50, 1, 200)
		speedacc.speed = 0
	End Method
End Type

Type Acceleration
	Field maximum: Float, minimum: Float
	Field speed: Float
	Field add: Float
	Field sub: Float
	Field ntns: Float
	Method New()
		maximum = 10
		minimum = 0
		speed = 0
		add = 1
		sub = 2
	End Method
	Method speedup(time: Float)
		speed = speed + add*time
		If speed > maximum speed = maximum
	End Method
	Method speeddown(time: Float)
		speed = speed - sub*time
		If speed < minimum speed = minimum
	End Method
	Method neartonull(time: Float)
		If speed > 0
			speed = speed - ntns*time
			If speed < 0 speed = 0
		ElseIf speed < 0
			speed = speed + ntns*time
			If speed > 0 speed = 0
		End If
	End Method
End Type
