Type Spark Extends SpaceObject
	Field xd: Float, yd: Float
	Field r: int, g: int, b: int
	Field alpha: Float
	Field lifetime: Float
	Field xscale: Float, yscale: Float
	
	Method New()
		MainGame.solist.add(Self)
	End Method
	
	Method draw(p: Point, wscale: Float, wrotation: Float)
		SetBlend LIGHTBLEND
		SetScale xscale*wscale, yscale*wscale
		SetColor r, g, b
		SetAlpha alpha
		SetRotation rotation+wrotation
		
		DrawImage RessourceLoader.sparkimage, p.x+x*wscale, p.y+y*wscale
		
		SetRotation 0
		SetBlend SOLIDBLEND
		SetAlpha 1
		SetScale 1, 1
		SetColor 255, 255, 255
	End Method
	
	Method collisionPossible: Int(o: SpaceObject)
		Return True
	End Method
	
	Method move(time: Float)
		If alpha > 0
			x = x + xd*time
			y = y + yd*time

			alpha = alpha - lifetime*time
		Else
			health = 0
			collisionradius = 1
			collisiondamage = 0
		End if
	End Method
	
	Method destroy()
		alpha = 0
	End Method
	
	Method collided(damage: Float)'p: Point)
		
	End Method
	
	Method getType: Int()
		Return SOT_SPARK
	End Method
	
	Function create: Spark(x: Float, y: Float, xscale: Float, yscale: Float, r: int, g: int, b: int, lifetime: Float, rot: Float, speed: Float, collisionradius: Float = 0, collisiondamage: Float = 0, health: Float = 0)
		Local sp:Spark = New Spark
		
		sp.x = x
		sp.y = y
		
		sp.xscale = xscale
		sp.yscale = yscale
		
		sp.xd = Cos(rot) * speed
		sp.yd = Sin(rot) * speed
		
		sp.rotation = rot
		
		sp.lifetime = lifetime
		
		sp.r = r
		sp.g = g
		sp.b = b
		
		sp.collisiondamage = collisiondamage
		sp.collisionradius = collisionradius
		sp.health = health
		
		sp.alpha = 1
		
		Return sp
	End Function
End type

Type OwnerSpark Extends SpaceObject
	Field owner: SpaceObject
	Field xd: Float, yd: Float
	Field r: int, g: int, b: int
	Field alpha: Float
	Field lifetime: Float
	Field xscale: Float, yscale: Float
	
	Method New()
		MainGame.solist.add(Self)
	End Method
	
	Method draw(p: Point, wscale: Float, wrotation: Float)
		SetBlend LIGHTBLEND
		SetScale xscale*wscale, yscale*wscale
		SetColor r, g, b
		SetAlpha alpha
		SetRotation rotation+wrotation
		
		DrawImage RessourceLoader.sparkimage, p.x+x*wscale, p.y+y*wscale
		
		SetRotation 0
		SetBlend SOLIDBLEND
		SetAlpha 1
		SetScale 1, 1
		SetColor 255, 255, 255
	End Method
	
	Method collisionPossible: Int(o: SpaceObject)
		If o <> owner Then Return True
		Return False
	End Method
	
	Method move(time: Float)
		If alpha > 0
			x = x + xd*time
			y = y + yd*time

			alpha = alpha - lifetime*time
		Else
			health = 0
			collisionradius = 1
			collisiondamage = 0
		End if
	End Method
	
	Method destroy()
		alpha = 0
	End Method
	
	Method collided(damage: Float)'p: Point)
		
	End Method
	
	Method getType: Int()
		Return SOT_OWNERSPARK
	End Method
	
	Function create: OwnerSpark(owner: SpaceObject, x: Float, y: Float, xscale: Float, yscale: Float, r: int, g: int, b: int, lifetime: Float, rot: Float, speed: Float, collisionradius: Float = 0, collisiondamage: Float = 0, health: Float = 0)
		Local sp:OwnerSpark = New OwnerSpark
		
		sp.x = x
		sp.y = y
		
		sp.owner = owner
		
		sp.xscale = xscale
		sp.yscale = yscale
		
		sp.xd = Cos(rot) * speed
		sp.yd = Sin(rot) * speed
		
		sp.rotation = rot
		
		sp.lifetime = lifetime
		
		sp.r = r
		sp.g = g
		sp.b = b
		
		sp.collisiondamage = collisiondamage
		sp.collisionradius = collisionradius
		sp.health = health
		
		sp.alpha = 1
		
		Return sp
	End Function
End type
